'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _debug2 = require('debug');

var _debug3 = _interopRequireDefault(_debug2);

var _warning = require('warning');

var _warning2 = _interopRequireDefault(_warning);

var _MemoryCacheStore = require('../cache/MemoryCacheStore');

var _MemoryCacheStore2 = _interopRequireDefault(_MemoryCacheStore);

var _CacheBasedSessionStore = require('../session/CacheBasedSessionStore');

var _CacheBasedSessionStore2 = _interopRequireDefault(_CacheBasedSessionStore);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug3.default)('core/bot/Bot');

var MINUTES_IN_ONE_YEAR = 365 * 24 * 60;

function createMemorySessionStore() {
  var cache = new _MemoryCacheStore2.default(500);
  return new _CacheBasedSessionStore2.default(cache, MINUTES_IN_ONE_YEAR);
}

var Bot = function () {
  function Bot(_ref) {
    var connector = _ref.connector,
        _ref$sessionStore = _ref.sessionStore,
        sessionStore = _ref$sessionStore === undefined ? createMemorySessionStore() : _ref$sessionStore;
    (0, _classCallCheck3.default)(this, Bot);
    this._contextExtensions = [];

    this._sessions = sessionStore;
    this._initialized = false;
    this._connector = connector;
    this._handler = null;
  }

  (0, _createClass3.default)(Bot, [{
    key: 'onEvent',
    value: function onEvent(handler) {
      this._handler = handler;
    }
  }, {
    key: 'handle',
    value: function handle(handler) {
      (0, _warning2.default)(false, '`handle` is deprecated. Use `onEvent` instead.');
      this.onEvent(handler);
    }
  }, {
    key: 'extendContext',
    value: function extendContext(fn) {
      this._contextExtensions.push(fn);
      return this;
    }
  }, {
    key: 'setMessageDelay',
    value: function setMessageDelay(milliseconds) {
      return this.extendContext(function (context) {
        context.setMessageDelay(milliseconds);
      });
    }
  }, {
    key: 'createRequestHandler',
    value: function createRequestHandler() {
      var _this = this;

      if (this._handler == null) {
        throw new Error('Bot: Missing event handler function. You should assign it using onEvent(...)');
      }

      return function () {
        var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(body) {
          var platform, sessionId, sessionKey, session, data, events, contexts;
          return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  if (body) {
                    _context.next = 2;
                    break;
                  }

                  throw new Error('Bot.createRequestHandler: Missing argument.');

                case 2:

                  debug(JSON.stringify(body, null, 2));

                  if (_this._initialized) {
                    _context.next = 7;
                    break;
                  }

                  _context.next = 6;
                  return _this._sessions.init();

                case 6:
                  _this._initialized = true;

                case 7:
                  platform = _this._connector.platform;
                  sessionId = _this._connector.getUniqueSessionIdFromRequest(body);

                  // Create or retrieve session if possible

                  sessionKey = void 0;
                  session = void 0;

                  if (!sessionId) {
                    _context.next = 19;
                    break;
                  }

                  sessionKey = platform + ':' + sessionId;

                  _context.next = 15;
                  return _this._sessions.read(sessionKey);

                case 15:
                  data = _context.sent;

                  session = data || Object.create(null);

                  _context.next = 19;
                  return _this._connector.updateSession(session, body);

                case 19:
                  events = _this._connector.mapRequestToEvents(body);
                  contexts = events.map(function (event) {
                    return _this._connector.createContext({
                      event: event,
                      session: session
                    });
                  });

                  // Call all of extension functions before passing to handler.

                  contexts.forEach(function (context) {
                    _this._contextExtensions.forEach(function (ext) {
                      ext(context);
                    });
                  });

                  Promise.all(contexts.map(function (context) {
                    if (_this._handler == null) {
                      throw new Error('Bot: Missing event handler function. You should assign it using onEvent(...)');
                    }
                    return _this._handler(context);
                  })).then(function () {
                    if (session) {
                      // $FlowFixMe: suppressing this error until we can refactor
                      session.lastActivity = Date.now();
                      _this._sessions.write(sessionKey, session);
                    }
                  }).catch(console.error);

                case 23:
                case 'end':
                  return _context.stop();
              }
            }
          }, _callee, _this);
        }));

        return function (_x) {
          return _ref2.apply(this, arguments);
        };
      }();
    }
  }, {
    key: 'connector',
    get: function get() {
      return this._connector;
    }
  }, {
    key: 'sessions',
    get: function get() {
      return this._sessions;
    }
  }, {
    key: 'handler',
    get: function get() {
      return this._handler;
    }
  }]);
  return Bot;
}();

exports.default = Bot;