'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _messagingApiLine = require('messaging-api-line');

var _LINEContext = require('../context/LINEContext');

var _LINEContext2 = _interopRequireDefault(_LINEContext);

var _LINEEvent = require('../context/LINEEvent');

var _LINEEvent2 = _interopRequireDefault(_LINEEvent);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var LINEConnector = function () {
  function LINEConnector(_ref) {
    var accessToken = _ref.accessToken,
        channelSecret = _ref.channelSecret;
    (0, _classCallCheck3.default)(this, LINEConnector);

    this._client = _messagingApiLine.LINEClient.connect(accessToken, channelSecret);
  }

  (0, _createClass3.default)(LINEConnector, [{
    key: 'getUniqueSessionIdFromRequest',
    value: function getUniqueSessionIdFromRequest(body) {
      var source = body.events[0].source;

      if (source.type === 'user') {
        return source.userId;
      } else if (source.type === 'group') {
        return source.groupId;
      } else if (source.type === 'room') {
        return source.roomId;
      }
      throw new TypeError('LINEConnector: sender type should be one of user, group, room.');
    }

    // FIXME: handling different type session

  }, {
    key: 'updateSession',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(session, body) {
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (session.type) {
                  _context.next = 4;
                  break;
                }

                _context.next = 3;
                return this._updateSession(session, body);

              case 3:
                return _context.abrupt('return');

              case 4:
                if (!(session.type === 'user')) {
                  _context.next = 9;
                  break;
                }

                if (session.user) {
                  _context.next = 8;
                  break;
                }

                _context.next = 8;
                return this._updateSession(session, body);

              case 8:
                return _context.abrupt('return');

              case 9:
                _context.next = 11;
                return this._updateSession(session, body);

              case 11:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function updateSession(_x, _x2) {
        return _ref2.apply(this, arguments);
      }

      return updateSession;
    }()
  }, {
    key: 'mapRequestToEvents',
    value: function mapRequestToEvents(body) {
      return body.events.map(function (e) {
        return new _LINEEvent2.default(e);
      });
    }
  }, {
    key: 'createContext',
    value: function createContext(_ref3) {
      var event = _ref3.event,
          session = _ref3.session;

      return new _LINEContext2.default({
        client: this._client,
        event: event,
        session: session
      });
    }
  }, {
    key: 'verifySignature',
    value: function verifySignature(rawBody, signature) {
      return this._client.isValidSignature(rawBody, signature);
    }
  }, {
    key: '_updateSession',
    value: function () {
      var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(session, body) {
        var source, memberIds, _memberIds, user;

        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                source = body.events[0].source;


                if (!session.type) {
                  session.type = source.type;
                }

                if (!(source.type === 'group')) {
                  _context2.next = 9;
                  break;
                }

                _context2.next = 5;
                return this._client.getAllGroupMemberIds(source.groupId);

              case 5:
                memberIds = _context2.sent;

                session.group = {
                  id: source.groupId,
                  members: memberIds.map(function (id) {
                    return { id: id };
                  })
                };
                _context2.next = 14;
                break;

              case 9:
                if (!(source.type === 'room')) {
                  _context2.next = 14;
                  break;
                }

                _context2.next = 12;
                return this._client.getAllRoomMemberIds(source.roomId);

              case 12:
                _memberIds = _context2.sent;

                session.room = {
                  id: source.roomId,
                  members: _memberIds.map(function (id) {
                    return { id: id };
                  })
                };

              case 14:
                if (!source.userId) {
                  _context2.next = 21;
                  break;
                }

                _context2.next = 17;
                return this._client.getUserProfile(source.userId);

              case 17:
                user = _context2.sent;

                session.user = (0, _extends3.default)({
                  id: source.userId,
                  platform: 'line'
                }, user);
                _context2.next = 22;
                break;

              case 21:
                session.user = null;

              case 22:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function _updateSession(_x3, _x4) {
        return _ref4.apply(this, arguments);
      }

      return _updateSession;
    }()
  }, {
    key: 'platform',
    get: function get() {
      return 'line';
    }
  }, {
    key: 'client',
    get: function get() {
      return this._client;
    }
  }]);
  return LINEConnector;
}(); /*
       eslint-disable class-methods-use-this
       
     */


exports.default = LINEConnector;