'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = generateIndexFile;
var requireServerLine = function requireServerLine(server) {
  switch (server) {
    case 'micro':
      return 'const { createRequestHandler } = require(\'toolbot-core-experiment/' + server + '\');\n\n';
    case undefined:
      // platform is console
      return '';
    default:
      return 'const { createServer } = require(\'toolbot-core-experiment/' + server + '\');\n\n';
  }
};

var requireBotJSONLine = function requireBotJSONLine(platform) {
  switch (platform) {
    case 'line':
      return 'const config = require(\'./bot.json\').LINE;\n';
    case 'console':
      return '';
    default:
      return 'const config = require(\'./bot.json\').' + platform + ';\n';
  }
};

var newBotLines = function newBotLines(bot, platform, sessionStore) {
  var lines = [];
  if (platform === 'console') {
    lines.push('const bot = new ' + bot + '();');
  } else {
    lines.push('const bot = new ' + bot + '({');
    lines.push('  accessToken: config.accessToken,');
    if (platform === 'line') {
      lines.push('  channelSecret: config.channelSecret,');
    } else if (platform === 'messenger') {
      lines.push('  appSecret: config.appSecret,');
    }
    if (sessionStore) {
      lines.push('  ' + sessionStore);
    }
    lines.push('});');
  }

  return lines.join('\n');
};

var runBotLines = function runBotLines(platform, server) {
  if (platform === 'console') {
    return 'bot.createRuntime();';
  } else if (server === 'micro') {
    return 'module.exports = createRequestHandler(bot);';
  }
  return 'const server = createServer(bot);\n\nserver.listen(5000, () => {\n  console.log(\'server is running...\');\n});';
};

function generateIndexFile(answer) {
  var platform = answer.platform,
      session = answer.session,
      server = answer.server;

  var dependencies = [];
  var sessionStore = '';

  switch (platform) {
    case 'line':
      dependencies.push('LINEBot');
      break;
    case 'slack':
      dependencies.push('SlackBot');
      break;
    case 'telegram':
      dependencies.push('TelegramBot');
      break;
    case 'console':
      dependencies.push('ConsoleBot');
      break;
    case 'messenger':
    default:
      dependencies.push('MessengerBot');
      break;
  }

  switch (session) {
    case 'file':
      dependencies.push('FileSessionStore');
      sessionStore = 'sessionStore: new FileSessionStore(),';
      break;
    case 'redis':
      dependencies.push('RedisCacheStore', 'CacheBasedSessionStore');
      sessionStore = 'sessionStore: new CacheBasedSessionStore(cache),';
      break;
    case 'mongo':
      dependencies.push('MongoSessionStore');
      sessionStore = 'sessionStore: new MongoSessionStore(\'mongodb://localhost:27017/\'),';
      break;
    default:
  }

  return 'const { ' + dependencies.join(', ') + ' } = require(\'toolbot-core-experiment\');\n' + requireServerLine(server) + requireBotJSONLine(platform) + (session === 'redis' ? '\nconst cache = new RedisCacheStore();\n' : '') + '\n' + newBotLines(dependencies[0], platform, sessionStore) + '\n\nbot.onEvent(context => {\n  context.sendText(\'Hello World\');\n});\n\n' + runBotLines(platform, server) + '\n';
}