'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _child_process = require('child_process');

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _validateNpmPackageName = require('validate-npm-package-name');

var _validateNpmPackageName2 = _interopRequireDefault(_validateNpmPackageName);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _crossSpawn = require('cross-spawn');

var _crossSpawn2 = _interopRequireDefault(_crossSpawn);

var _log = require('../../shared/log');

var _generateIndexFile = require('./generateIndexFile');

var _generateIndexFile2 = _interopRequireDefault(_generateIndexFile);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var questions = [{
  name: 'name',
  message: "What's your project name?",
  type: 'input'
}, {
  name: 'platform',
  message: 'What platform of bot do you want to create?',
  type: 'list',
  choices: ['messenger', 'line', 'slack', 'telegram', 'console']
}, {
  name: 'session',
  message: 'Where do you want to store session?',
  type: 'list',
  choices: ['memory', 'file', 'redis', 'mongo']
}, {
  when: function when(answer) {
    return answer.platform !== 'console';
  },
  name: 'server',
  message: 'What kind of server do you want to use?',
  type: 'list',
  choices: ['express', 'koa', 'micro', 'restify']
}];

var generateBotJson = function generateBotJson(platform) {
  var accessToken = '__PUT_YOUR_ACCESS_TOKEN_HERE__';
  switch (platform) {
    case 'slack':
      return {
        slack: {
          accessToken: accessToken
        }
      };
    case 'line':
      return {
        LINE: {
          accessToken: accessToken,
          channelSecret: '__PUT_YOUR_CHANNEL_SECRET_HERE__'
        }
      };
    case 'telegram':
      return {
        telegram: {
          accessToken: accessToken
        }
      };
    case 'messenger':
    default:
      return {
        messenger: {
          accessToken: accessToken,
          verifyToken: '__PUT_YOUR_VERITY_TOKEN_HERE__',
          appId: '__PUT_YOUR_APP_ID_HERE__',
          appSecret: '__PUT_YOUR_APP_SECRET_HERE__'
        }
      };
  }
};

var shouldUseYarn = function shouldUseYarn() {
  try {
    (0, _child_process.execSync)('yarnpkg --version', { stdio: 'ignore' });
    return true;
  } catch (e) {
    return false;
  }
};

var isSafeToCreateProjectIn = function isSafeToCreateProjectIn(root, name) {
  var validFiles = ['.DS_Store', 'Thumbs.db', '.git', '.gitignore', '.idea', 'README.md', 'LICENSE', 'web.iml', '.hg', '.hgignore', '.hgcheck'];
  (0, _log.print)('');

  var conflicts = _fsExtra2.default.readdirSync(root).filter(function (file) {
    return !validFiles.includes(file);
  });

  if (conflicts.length < 1) {
    return true;
  }

  (0, _log.print)('The directory ' + name + ' contains files that could conflict:');
  (0, _log.print)('');

  // eslint-disable-next-line no-restricted-syntax
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = conflicts[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var file = _step.value;

      (0, _log.print)('  ' + file);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  (0, _log.print)('');
  (0, _log.print)('Either try using a new directory name, or remove the files listed above.');

  return false;
};

var printValidationResults = function printValidationResults(results) {
  if (typeof results !== 'undefined') {
    results.forEach(function (Error) {
      (0, _log.error)('  *  ' + Error);
    });
  }
};

var checkBotName = function checkBotName(botName) {
  var validationResult = (0, _validateNpmPackageName2.default)(botName);
  if (!validationResult.validForNewPackages) {
    (0, _log.error)('Could not create a project called "' + botName + '" because of npm naming restrictions:');
    printValidationResults(validationResult.errors);
    printValidationResults(validationResult.warnings);
    return false;
  }
  return true;
};

var install = function install(useYarn, allDependencies) {
  return new Promise(function (resolve, reject) {
    var command = void 0;
    var args = [];
    if (useYarn) {
      command = 'yarnpkg';
      args = args.concat(['add', '--exact'], allDependencies.dependencies);
      (0, _crossSpawn2.default)(command, args, { stdio: 'inherit' });
      args = [];
      args = args.concat(['add', '--dev'], allDependencies.devDependencies);
    } else {
      command = 'npm';
      args = args.concat(['install', '--save', '--save-exact', '--loglevel', 'error'], allDependencies.dependencies);
      (0, _crossSpawn2.default)(command, args, { stdio: 'inherit' });
      args = [];
      args = args.concat(['install', '--dev', '--save-exact', '--loglevel', 'error'], allDependencies.devDependencies);
    }

    var child = (0, _crossSpawn2.default)(command, args, { stdio: 'inherit' });
    child.on('close', function (code) {
      if (code !== 0) {
        reject({
          command: command + ' ' + args.join(' ')
        });
        return;
      }
      resolve();
    });
  });
};

var run = function () {
  var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(root, botName, answer, useYarn) {
    var allDependencies, indexFile, botJson, knownGeneratedFiles, currentFiles, remainingFiles;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.prev = 0;
            allDependencies = {
              dependencies: ['babel-register', 'toolbot-core-experiment'],
              devDependencies: ['nodemon']
            };

            (0, _log.print)('Installing packages... This might take a couple of minutes.');
            (0, _log.print)('');

            _context.next = 6;
            return install(useYarn, allDependencies);

          case 6:
            indexFile = (0, _generateIndexFile2.default)(answer);

            _fsExtra2.default.writeFileSync(_path2.default.join(root, 'index.js'), indexFile);

            if (answer.platform !== 'console') {
              botJson = generateBotJson(answer.platform);

              _fsExtra2.default.writeFileSync(_path2.default.join(root, 'bot.json'), JSON.stringify(botJson, null, 2));
              (0, _log.print)('\n\nAlready creat ' + (0, _log.bold)('bot.json') + ' for you, please make sure you have edited it before run the bot.\n\n');
            }
            _context.next = 24;
            break;

          case 11:
            _context.prev = 11;
            _context.t0 = _context['catch'](0);

            (0, _log.print)('');
            (0, _log.print)('Aborting installation.');
            if (_context.t0.command) {
              (0, _log.error)('  ' + _context.t0.command + ' has failed.');
            } else {
              (0, _log.error)('Unexpected error. Please report it as a bug:');
              (0, _log.print)(_context.t0);
            }
            (0, _log.print)('');

            knownGeneratedFiles = ['package.json', 'npm-debug.log', 'yarn-error.log', 'yarn-debug.log', 'node_modules'];
            currentFiles = _fsExtra2.default.readdirSync(_path2.default.join(root));

            currentFiles.forEach(function (file) {
              knownGeneratedFiles.forEach(function (fileToMatch) {
                if (fileToMatch.match(/.log/g) && file.indexOf(fileToMatch) === 0 || file === fileToMatch) {
                  (0, _log.print)('Deleting generated file... ' + file);
                  _fsExtra2.default.removeSync(_path2.default.join(root, file));
                }
              });
            });
            remainingFiles = _fsExtra2.default.readdirSync(_path2.default.join(root));

            if (!remainingFiles.length) {
              (0, _log.print)('Deleting ' + botName + ' / from ' + _path2.default.resolve(root, '..'));
              process.chdir(_path2.default.resolve(root, '..'));
              _fsExtra2.default.removeSync(_path2.default.join(root));
            }
            (0, _log.print)('Done.');
            return _context.abrupt('return', process.exit(1));

          case 24:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined, [[0, 11]]);
  }));

  return function run(_x, _x2, _x3, _x4) {
    return _ref.apply(this, arguments);
  };
}();

var createBot = function () {
  var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(answer) {
    var name, platform, root, botName, packageJson, useYarn;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            name = answer.name, platform = answer.platform;
            root = _path2.default.resolve(name);
            botName = _path2.default.basename(root);

            if (checkBotName(botName)) {
              _context2.next = 5;
              break;
            }

            return _context2.abrupt('return', process.exit(1));

          case 5:
            _fsExtra2.default.ensureDirSync(name);

            if (isSafeToCreateProjectIn(root, name)) {
              _context2.next = 8;
              break;
            }

            return _context2.abrupt('return', process.exit(1));

          case 8:

            (0, _log.print)('Creating a new ' + platform + ' bot in ' + root + '.');
            (0, _log.print)('');

            packageJson = {
              name: botName,
              version: '0.1.0',
              private: true,
              scripts: {
                dev: 'nodemon index.js',
                start: 'node index.js'
              }
            };

            _fsExtra2.default.writeFileSync(_path2.default.join(root, 'package.json'), JSON.stringify(packageJson, null, 2));

            process.chdir(root);

            useYarn = shouldUseYarn();
            _context2.next = 16;
            return run(root, botName, answer, useYarn);

          case 16:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  }));

  return function createBot(_x5) {
    return _ref2.apply(this, arguments);
  };
}();

exports.default = function () {
  var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3() {
    var answer, name;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.prev = 0;
            _context3.next = 3;
            return _inquirer2.default.prompt(questions);

          case 3:
            answer = _context3.sent;
            name = answer.name;

            if (!(typeof name === 'undefined' || name === '')) {
              _context3.next = 13;
              break;
            }

            (0, _log.print)('');
            (0, _log.error)('Please specify the project name');
            (0, _log.print)('For example:');
            (0, _log.print)("  ? What's your project name? my-bot");
            (0, _log.print)('');
            (0, _log.print)("Run 'tce --help' to see all options.");
            return _context3.abrupt('return', process.exit(1));

          case 13:
            _context3.next = 15;
            return createBot(answer);

          case 15:
            _context3.next = 22;
            break;

          case 17:
            _context3.prev = 17;
            _context3.t0 = _context3['catch'](0);

            (0, _log.error)('init error with');
            if (_context3.t0.response) {
              (0, _log.error)('status: ' + _context3.t0.response.status);
            } else {
              (0, _log.error)('message: ' + _context3.t0.message);
            }
            return _context3.abrupt('return', process.exit(1));

          case 22:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this, [[0, 17]]);
  }));

  function init() {
    return _ref3.apply(this, arguments);
  }

  return init;
}();