'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.localClient = exports.client = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _axios = require('axios');

var _axios2 = _interopRequireDefault(_axios);

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _getConfig = require('../shared/getConfig');

var _getConfig2 = _interopRequireDefault(_getConfig);

var _log = require('../shared/log');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var client = exports.client = _axios2.default.create({
  baseURL: 'https://graph.facebook.com/v2.9/',
  headers: { 'Content-Type': 'application/json' }
});

var localClient = exports.localClient = _axios2.default.create({
  baseURL: 'http://localhost:4040'
});

var getWebhookFromNgrok = function () {
  var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
    var res;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return localClient.get('/api/tunnels');

          case 2:
            res = _context.sent;
            return _context.abrupt('return', (0, _get2.default)(res, 'data.tunnels[1].public_url'));

          case 4:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function getWebhookFromNgrok() {
    return _ref.apply(this, arguments);
  };
}();

var _getClientToken = function () {
  var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(clientId, clientSecret) {
    var res;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return client.get('/oauth/access_token?client_id=' + clientId + '&client_secret=' + clientSecret + '&grant_type=client_credentials');

          case 2:
            res = _context2.sent;
            return _context2.abrupt('return', res.data.access_token);

          case 4:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  }));

  return function _getClientToken(_x, _x2) {
    return _ref2.apply(this, arguments);
  };
}();

exports.default = function () {
  var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(_webhook, _configPath, _verifyToken) {
    var platform, configPath, config, defaultFields, webhook, verifyToken, fields, token, res;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.prev = 0;
            platform = 'messenger';
            configPath = _configPath || 'bot.json';
            config = (0, _getConfig2.default)(configPath, platform);
            defaultFields = ['messages', 'messaging_postbacks', 'messaging_referrals'];
            _context3.t0 = _webhook;

            if (_context3.t0) {
              _context3.next = 10;
              break;
            }

            _context3.next = 9;
            return getWebhookFromNgrok();

          case 9:
            _context3.t0 = _context3.sent;

          case 10:
            webhook = _context3.t0;
            verifyToken = _verifyToken || config.verifyToken;


            (0, _invariant2.default)(config.appId, '`appId` is not found in config file');
            (0, _invariant2.default)(config.appSecret, '`appSecret` is not found in config file');
            (0, _invariant2.default)(verifyToken, 'verifyToken is required but not found. using -v <verifyToken> to setup or list `verifyToken` key it in config file.');
            (0, _invariant2.default)(webhook, '`webhook` is required but not found. Use -w <webhook> to setup or make sure you are running ngrok server.');

            if (!config.fields) {
              (0, _log.print)('`fields` is not found in config file, we will use ' + (0, _log.bold)(defaultFields.join(', ')) + ' to setup.');
              (0, _log.print)('See more on: https://developers.facebook.com/docs/graph-api/reference/v2.9/app/subscriptions');
            }
            fields = config.fields || defaultFields;
            _context3.next = 20;
            return _getClientToken(config.appId, config.appSecret);

          case 20:
            token = _context3.sent;
            _context3.next = 23;
            return client.post('/' + config.appId + '/subscriptions?access_token=' + token, {
              object: 'page',
              callback_url: webhook,
              verify_token: verifyToken,
              fields: fields
            });

          case 23:
            res = _context3.sent;

            (0, _invariant2.default)(res.data.success, 'Setting for webhook is failed');

            (0, _log.print)('Successfully set webhook');
            _context3.next = 33;
            break;

          case 28:
            _context3.prev = 28;
            _context3.t1 = _context3['catch'](0);

            (0, _log.error)('set webhook error with');
            if (_context3.t1.response) {
              (0, _log.error)('status: ' + (0, _log.bold)(_context3.t1.response.status));
              if (_context3.t1.response.data) {
                (0, _log.error)('data: ' + (0, _log.bold)(JSON.stringify(_context3.t1.response.data, null, 2)));
              }
            } else {
              (0, _log.error)('message: ' + (0, _log.bold)(_context3.t1.message));
            }
            return _context3.abrupt('return', process.exit(1));

          case 33:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this, [[0, 28]]);
  }));

  function setWebhook(_x3, _x4, _x5) {
    return _ref3.apply(this, arguments);
  }

  return setWebhook;
}();