'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _delay = require('delay');

var _delay2 = _interopRequireDefault(_delay);

var _warning = require('warning');

var _warning2 = _interopRequireDefault(_warning);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _messagingApiLine = require('messaging-api-line');

var _LINEEvent = require('./LINEEvent');

var _LINEEvent2 = _interopRequireDefault(_LINEEvent);

var _DelayableJobQueue = require('./DelayableJobQueue');

var _DelayableJobQueue2 = _interopRequireDefault(_DelayableJobQueue);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var LINEContext = function () {
  function LINEContext(_ref) {
    var client = _ref.client,
        event = _ref.event,
        session = _ref.session;
    (0, _classCallCheck3.default)(this, LINEContext);
    this._messageDelay = 1000;
    this._replied = false;

    this._client = client;
    this._event = event;
    this._session = session;
    this._jobQueue = new _DelayableJobQueue2.default();
    this._jobQueue.beforeEach(function (_ref2) {
      var delay = _ref2.delay;
      return (0, _delay2.default)(delay);
    });
  }

  /**
   * The name of the platform.
   *
   */


  (0, _createClass3.default)(LINEContext, [{
    key: 'setMessageDelay',


    /**
     * Set delay before sending every messages.
     *
     */
    value: function setMessageDelay(milliseconds) {
      this._messageDelay = milliseconds;
    }

    /**
     * Delay and show indicators for milliseconds.
     *
     */

  }, {
    key: 'typing',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(milliseconds) {
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return (0, _delay2.default)(milliseconds);

              case 2:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function typing(_x) {
        return _ref3.apply(this, arguments);
      }

      return typing;
    }()

    /**
     * Send text to the owner of then session.
     *
     */

  }, {
    key: 'sendText',
    value: function sendText(text) {
      if (!this._session) {
        (0, _warning2.default)(false, 'sendText: should not be called in context without session');
        return Promise.resolve();
      }
      return this._enqueue({
        instance: this._client,
        method: 'pushText',
        args: [this._session.user.id, text],
        delay: this._messageDelay,
        showIndicators: true
      });
    }
  }, {
    key: 'sendTextWithDelay',
    value: function sendTextWithDelay(delay, text) {
      if (!this._session) {
        (0, _warning2.default)(false, 'sendTextWithDelay: should not be called in context without session');
        return Promise.resolve();
      }
      return this._enqueue({
        instance: this._client,
        method: 'pushText',
        args: [this._session.user.id, text],
        delay: delay,
        showIndicators: true
      });
    }
  }, {
    key: '_enqueue',
    value: function _enqueue(job) {
      var _this = this;

      return new Promise(function (resolve, reject) {
        _this._jobQueue.enqueue((0, _extends3.default)({}, job, {
          onSuccess: resolve,
          onError: reject
        }));
      });
    }
  }, {
    key: 'platform',
    get: function get() {
      return 'line';
    }

    /**
     * The client instance.
     *
     */

  }, {
    key: 'client',
    get: function get() {
      return this._client;
    }

    /**
     * The event instance.
     *
     */

  }, {
    key: 'event',
    get: function get() {
      return this._event;
    }

    /**
     * The session state of the context.
     *
     */

  }, {
    key: 'session',
    get: function get() {
      return this._session;
    }

    /**
     * Determine if the reply token is already used.
     *
     */

  }, {
    key: 'replied',
    get: function get() {
      return this._replied;
    }
  }]);
  return LINEContext;
}();

var types = ['Text', 'Image', 'Video', 'Audio', 'Location', 'Sticker', 'Imagemap', 'ButtonTemplate', 'ConfirmTemplate', 'CarouselTemplate', 'ImageCarouselTemplate'];
types.forEach(function (type) {
  Object.defineProperty(LINEContext.prototype, 'reply' + type, {
    enumerable: false,
    configurable: true,
    writable: true,
    value: function value() {
      (0, _invariant2.default)(!this._replied, 'Can not reply event mulitple times');

      this._replied = true;

      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      return this._enqueue({
        instance: this._client,
        method: 'reply' + type,
        args: [this._event.replyToken].concat(args),
        delay: this._messageDelay,
        showIndicators: true
      });
    }
  });

  Object.defineProperty(LINEContext.prototype, 'push' + type, {
    enumerable: false,
    configurable: true,
    writable: true,
    value: function value() {
      if (!this._session) {
        (0, _warning2.default)(false, 'push' + type + ': should not be called in context without session');
        return Promise.resolve();
      }

      for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      return this._enqueue({
        instance: this._client,
        method: 'push' + type,
        args: [this._session.user.id].concat(args),
        delay: this._messageDelay,
        showIndicators: true
      });
    }
  });
});

types.filter(function (type) {
  return type !== 'Text';
}).forEach(function (type) {
  Object.defineProperty(LINEContext.prototype, 'send' + type, {
    enumerable: false,
    configurable: true,
    writable: true,
    value: function value() {
      if (!this._session) {
        (0, _warning2.default)(false, 'send' + type + ': should not be called in context without session');
        return Promise.resolve();
      }

      for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
        args[_key3] = arguments[_key3];
      }

      return this._enqueue({
        instance: this._client,
        method: 'push' + type,
        args: [this._session.user.id].concat(args),
        delay: this._messageDelay,
        showIndicators: true
      });
    }
  });

  Object.defineProperty(LINEContext.prototype, 'send' + type + 'WithDelay', {
    enumerable: false,
    configurable: true,
    writable: true,
    value: function value(delay) {
      (0, _warning2.default)(false, 'send' + type + 'WithDelay is deprecated.');

      if (!this._session) {
        (0, _warning2.default)(false, 'send' + type + 'WithDelay: should not be called in context without session');
        return Promise.resolve();
      }

      for (var _len4 = arguments.length, rest = Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
        rest[_key4 - 1] = arguments[_key4];
      }

      return this._enqueue({
        instance: this._client,
        method: 'push' + type,
        args: [this._session.user.id].concat(rest),
        delay: delay,
        showIndicators: true
      });
    }
  });
});

exports.default = LINEContext;