'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var LINEEvent = function () {
  function LINEEvent(rawEvent) {
    (0, _classCallCheck3.default)(this, LINEEvent);

    this._rawEvent = rawEvent;
  }

  /**
   * Underlying raw event from LINE.
   *
   */


  (0, _createClass3.default)(LINEEvent, [{
    key: 'rawEvent',
    get: function get() {
      return this._rawEvent;
    }

    /**
     * The reply token from LINE raw event. Only present on message, follow, join, postback, beacon events.
     *
     */

  }, {
    key: 'replyToken',
    get: function get() {
      return this._rawEvent.replyToken || null;
    }

    /**
     * Determine if the event is a message event.
     *
     */

  }, {
    key: 'isMessage',
    get: function get() {
      return this._rawEvent.type === 'message';
    }

    /**
     * The message object from LINE raw event.
     *
     */

  }, {
    key: 'message',
    get: function get() {
      return this._rawEvent.message;
    }

    /**
     * Determine if the event is a message event which includes text.
     *
     */

  }, {
    key: 'isTextMessage',
    get: function get() {
      return this.isMessage && this.message.type === 'text';
    }

    /**
     * Determine if the event is a message event which includes image.
     *
     */

  }, {
    key: 'isImageMessage',
    get: function get() {
      return this.isMessage && this.message.type === 'image';
    }

    /**
     * Determine if the event is a message event which includes video.
     *
     */

  }, {
    key: 'isVideoMessage',
    get: function get() {
      return this.isMessage && this.message.type === 'video';
    }

    /**
     * Determine if the event is a message event which includes audio.
     *
     */

  }, {
    key: 'isAudioMessage',
    get: function get() {
      return this.isMessage && this.message.type === 'audio';
    }

    /**
     * Determine if the event is a message event which includes location.
     *
     */

  }, {
    key: 'isLocationMessage',
    get: function get() {
      return this.isMessage && this.message.type === 'location';
    }

    /**
     * Determine if the event is a message event which includes sticker.
     *
     */

  }, {
    key: 'isStickerMessage',
    get: function get() {
      return this.isMessage && this.message.type === 'sticker';
    }

    /**
     * Determine if the event is a follow event.
     *
     */

  }, {
    key: 'isFollow',
    get: function get() {
      return this._rawEvent.type === 'follow';
    }

    /**
     * Determine if the event is an unfollow event.
     *
     */

  }, {
    key: 'isUnfollow',
    get: function get() {
      return this._rawEvent.type === 'unfollow';
    }

    /**
     * Determine if the event is a join event.
     *
     */

  }, {
    key: 'isJoin',
    get: function get() {
      return this._rawEvent.type === 'join';
    }

    /**
     * Determine if the event is a leave event.
     *
     */

  }, {
    key: 'isLeave',
    get: function get() {
      return this._rawEvent.type === 'leave';
    }

    /**
     * Determine if the event is a postback event.
     *
     */

  }, {
    key: 'isPostback',
    get: function get() {
      return this._rawEvent.type === 'postback';
    }

    /**
     * The postback object from LINE raw event.
     *
     */

  }, {
    key: 'postback',
    get: function get() {
      return this._rawEvent.postback || null;
    }

    /**
     * The date string from LINE postback event.
     *
     */

  }, {
    key: 'date',
    get: function get() {
      if (this.postback && this.postback.params && this.postback.params.date) {
        return this.postback.params.date;
      }
      return null;
    }

    /**
     * The time string from LINE postback event.
     *
     */

  }, {
    key: 'time',
    get: function get() {
      if (this.postback && this.postback.params && this.postback.params.time) {
        return this.postback.params.time;
      }
      return null;
    }

    /**
     * The datetime string from LINE postback event.
     *
     */

  }, {
    key: 'datetime',
    get: function get() {
      if (this.postback && this.postback.params && this.postback.params.datetime) {
        return this.postback.params.datetime;
      }
      return null;
    }

    /**
     * Determine if the event is a beacon event.
     *
     */

  }, {
    key: 'isBeacon',
    get: function get() {
      return this._rawEvent.type === 'beacon';
    }
  }]);
  return LINEEvent;
}();

exports.default = LINEEvent;