'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _delay = require('delay');

var _delay2 = _interopRequireDefault(_delay);

var _warning = require('warning');

var _warning2 = _interopRequireDefault(_warning);

var _messagingApiMessenger = require('messaging-api-messenger');

var _MessengerEvent = require('./MessengerEvent');

var _MessengerEvent2 = _interopRequireDefault(_MessengerEvent);

var _DelayableJobQueue = require('./DelayableJobQueue');

var _DelayableJobQueue2 = _interopRequireDefault(_DelayableJobQueue);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var MessengerContext = function () {
  function MessengerContext(_ref) {
    var _this = this;

    var client = _ref.client,
        event = _ref.event,
        session = _ref.session;
    (0, _classCallCheck3.default)(this, MessengerContext);
    this._messageDelay = 1000;

    this._client = client;
    this._event = event;
    this._session = session;
    this._jobQueue = new _DelayableJobQueue2.default();
    this._jobQueue.beforeEach(function () {
      var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(_ref3) {
        var delay = _ref3.delay,
            _ref3$showIndicators = _ref3.showIndicators,
            showIndicators = _ref3$showIndicators === undefined ? true : _ref3$showIndicators;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (showIndicators) {
                  _this.typingOn();
                }
                _context.next = 3;
                return (0, _delay2.default)(delay);

              case 3:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, _this);
      }));

      return function (_x) {
        return _ref2.apply(this, arguments);
      };
    }());
    this._jobQueue.after(function () {
      var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(_ref5) {
        var _ref5$showIndicators = _ref5.showIndicators,
            showIndicators = _ref5$showIndicators === undefined ? true : _ref5$showIndicators;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (showIndicators) {
                  _this.typingOff();
                }

              case 1:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, _this);
      }));

      return function (_x2) {
        return _ref4.apply(this, arguments);
      };
    }());
  }

  /**
   * The name of the platform.
   *
   */


  (0, _createClass3.default)(MessengerContext, [{
    key: 'setMessageDelay',


    /**
     * Set delay before sending every messages.
     *
     */
    value: function setMessageDelay(milliseconds) {
      this._messageDelay = milliseconds;
    }

    /**
     * Delay and show indicators for milliseconds.
     *
     */

  }, {
    key: 'typing',
    value: function () {
      var _ref6 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(milliseconds) {
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return this.typingOn();

              case 2:
                _context3.next = 4;
                return (0, _delay2.default)(milliseconds);

              case 4:
                _context3.next = 6;
                return this.typingOff();

              case 6:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function typing(_x3) {
        return _ref6.apply(this, arguments);
      }

      return typing;
    }()

    /**
     * Send text to the owner of then session.
     *
     */

  }, {
    key: 'sendText',
    value: function sendText(text, options) {
      if (!this._session) {
        (0, _warning2.default)(false, 'sendText: should not be called in context without session');
        return Promise.resolve();
      }
      return this._enqueue({
        instance: this._client,
        method: 'sendText',
        args: options ? [this._session.user.id, text, options] : [this._session.user.id, text],
        delay: this._messageDelay,
        showIndicators: true
      });
    }
  }, {
    key: 'sendTextWithDelay',
    value: function sendTextWithDelay(delay, text) {
      if (!this._session) {
        (0, _warning2.default)(false, 'sendTextWithDelay: should not be called in context without session');
        return Promise.resolve();
      }
      return this._enqueue({
        instance: this._client,
        method: 'sendText',
        args: [this._session.user.id, text],
        delay: delay,
        showIndicators: true
      });
    }
  }, {
    key: 'typingOn',
    value: function typingOn() {
      if (!this._session) {
        (0, _warning2.default)(false, 'typingOn: should not be called in context without session');
        return Promise.resolve();
      }
      return this._client.typingOn(this._session.user.id);
    }
  }, {
    key: 'typingOff',
    value: function typingOff() {
      if (!this._session) {
        (0, _warning2.default)(false, 'typingOff: should not be called in context without session');
        return Promise.resolve();
      }
      return this._client.typingOff(this._session.user.id);
    }
  }, {
    key: 'turnTypingIndicatorsOn',
    value: function turnTypingIndicatorsOn() {
      (0, _warning2.default)(false, '`turnTypingIndicatorsOn` is deprecated. Use `typingOn` instead');
      if (!this._session) {
        (0, _warning2.default)(false, 'turnTypingIndicatorsOn: should not be called in context without session');
        return Promise.resolve();
      }
      return this._client.typingOn(this._session.user.id);
    }
  }, {
    key: 'turnTypingIndicatorsOff',
    value: function turnTypingIndicatorsOff() {
      (0, _warning2.default)(false, '`turnTypingIndicatorsOff` is deprecated. Use `typingOff` instead');
      if (!this._session) {
        (0, _warning2.default)(false, 'turnTypingIndicatorsOff: should not be called in context without session');
        return Promise.resolve();
      }
      return this._client.typingOff(this._session.user.id);
    }
  }, {
    key: '_enqueue',
    value: function _enqueue(job) {
      var _this2 = this;

      return new Promise(function (resolve, reject) {
        _this2._jobQueue.enqueue((0, _extends3.default)({}, job, {
          onSuccess: resolve,
          onError: reject
        }));
      });
    }
  }, {
    key: 'platform',
    get: function get() {
      return 'messenger';
    }

    /**
     * The client instance.
     *
     */

  }, {
    key: 'client',
    get: function get() {
      return this._client;
    }

    /**
     * The event instance.
     *
     */

  }, {
    key: 'event',
    get: function get() {
      return this._event;
    }

    /**
     * The session state of the context.
     *
     */

  }, {
    key: 'session',
    get: function get() {
      return this._session;
    }
  }]);
  return MessengerContext;
}();

var sendMethods = ['sendAttachment', 'sendImage', 'sendAudio', 'sendVideo', 'sendFile', 'sendQuickReplies', 'sendGenericTemplate', 'sendButtonTemplate', 'sendListTemplate', 'sendReceiptTemplate', 'sendAirlineBoardingPassTemplate', 'sendAirlineCheckinTemplate', 'sendAirlineItineraryTemplate', 'sendAirlineFlightUpdateTemplate'];

sendMethods.forEach(function (method) {
  Object.defineProperty(MessengerContext.prototype, '' + method, {
    enumerable: false,
    configurable: true,
    writable: true,
    value: function value() {
      if (!this._session) {
        (0, _warning2.default)(false, method + ': should not be called in context without session');
        return Promise.resolve();
      }

      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      return this._enqueue({
        instance: this._client,
        method: method,
        args: [this._session.user.id].concat(args),
        delay: this._messageDelay,
        showIndicators: true
      });
    }
  });

  Object.defineProperty(MessengerContext.prototype, method + 'WithDelay', {
    enumerable: false,
    configurable: true,
    writable: true,
    value: function value(delay) {
      (0, _warning2.default)(false, method + 'WithDelay is deprecated.');
      if (!this._session) {
        (0, _warning2.default)(false, method + 'WithDelay: should not be called in context without session');
        return Promise.resolve();
      }

      for (var _len2 = arguments.length, rest = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
        rest[_key2 - 1] = arguments[_key2];
      }

      return this._enqueue({
        instance: this._client,
        method: method,
        args: [this._session.user.id].concat(rest),
        delay: delay,
        showIndicators: true
      });
    }
  });
});

exports.default = MessengerContext;