'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var MessengerEvent = function () {
  function MessengerEvent(rawEvent) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    (0, _classCallCheck3.default)(this, MessengerEvent);

    this._rawEvent = rawEvent;
    this._isStandby = options.isStandby || false;
  }

  /**
   * Underlying raw event from Messenger.
   *
   */


  (0, _createClass3.default)(MessengerEvent, [{
    key: 'rawEvent',
    get: function get() {
      return this._rawEvent;
    }

    /**
     * Determine if the event is a message event.
     *
     */

  }, {
    key: 'isMessage',
    get: function get() {
      return !!this._rawEvent.message && (0, _typeof3.default)(this._rawEvent.message) === 'object';
    }

    /**
     * The message object from Messenger raw event.
     *
     */

  }, {
    key: 'message',
    get: function get() {
      return this._rawEvent.message;
    }

    /**
     * Determine if the event is a message event which includes text.
     *
     */

  }, {
    key: 'isTextMessage',
    get: function get() {
      return this.isMessage && typeof this.message.text === 'string';
    }

    /**
     * Determine if the event has any attachments.
     *
     */

  }, {
    key: 'hasAttachment',
    get: function get() {
      return this.isMessage && !!this.message.attachments && this.message.attachments.length > 0;
    }

    /**
     * The attachments array from Messenger raw event.
     *
     */

  }, {
    key: 'attachments',
    get: function get() {
      return this.message ? this.message.attachments : null;
    }

    /**
     * Determine if the event is a message event which includes image attachment.
     *
     */

  }, {
    key: 'isImageMessage',
    get: function get() {
      return this.hasAttachment && this.attachments[0].type === 'image';
    }

    /**
     * Determine if the event is a message event which includes audio attachment.
     *
     */

  }, {
    key: 'isAudioMessage',
    get: function get() {
      return this.hasAttachment && this.attachments[0].type === 'audio';
    }

    /**
     * Determine if the event is a message event which includes video attachment.
     *
     */

  }, {
    key: 'isVideoMessage',
    get: function get() {
      return this.hasAttachment && this.attachments[0].type === 'video';
    }

    /**
     * Determine if the event is a message event which includes location attachment.
     *
     */

  }, {
    key: 'isLocationMessage',
    get: function get() {
      return this.hasAttachment && this.attachments[0].type === 'location';
    }

    /**
     * Determine if the event is a message event which includes file attachment.
     *
     */

  }, {
    key: 'isFileMessage',
    get: function get() {
      return this.hasAttachment && this.attachments[0].type === 'file';
    }

    /**
     * Determine if the event is a message event which includes fallback attachment.
     *
     */

  }, {
    key: 'isFallbackMessage',
    get: function get() {
      return this.hasAttachment && this.attachments[0].type === 'fallback';
    }

    /**
     * Determine if the event is a message event which includes sticker.
     *
     */

  }, {
    key: 'isStickerMessage',
    get: function get() {
      return this.isMessage && typeof this.message.sticker_id === 'number';
    }

    /**
     * Determine if the event is a message event which includes 'like' sticker.
     *
     */

  }, {
    key: 'isLikeSticker',
    get: function get() {
      return this.isStickerMessage && (this.message.sticker_id === 369239263222822 || this.message.sticker_id === 369239343222814);
    }

    /**
     * Determine if the event is a message event triggered from quick reply.
     *
     */

  }, {
    key: 'isQuickReply',
    get: function get() {
      return this.isMessage && !!this.message.quick_reply && (0, _typeof3.default)(this.message.quick_reply) === 'object';
    }

    /**
     * The quick reply object from Messenger raw event.
     *
     */

  }, {
    key: 'quickReply',
    get: function get() {
      return this.message ? this.message.quick_reply : null;
    }

    /**
     * Determine if the event is a message event sent from our side.
     *
     */

  }, {
    key: 'isEcho',
    get: function get() {
      return this.isMessage && !!this.message.is_echo;
    }

    /**
     * Determine if the event is a postback event.
     *
     */

  }, {
    key: 'isPostback',
    get: function get() {
      return !!this._rawEvent.postback && (0, _typeof3.default)(this._rawEvent.postback) === 'object';
    }

    /**
     * The postback object from Messenger raw event.
     *
     */

  }, {
    key: 'postback',
    get: function get() {
      return this._rawEvent.postback || null;
    }

    /**
     * Determine if the event is a payment event.
     *
     */

  }, {
    key: 'isPayment',
    get: function get() {
      return !!this._rawEvent.payment && (0, _typeof3.default)(this._rawEvent.payment) === 'object';
    }

    /**
     * The payment object from Messenger raw event.
     *
     */

  }, {
    key: 'payment',
    get: function get() {
      return this._rawEvent.payment || null;
    }

    /**
     * Determine if the event is a message read event.
     *
     */

  }, {
    key: 'isRead',
    get: function get() {
      return !!this._rawEvent.read && (0, _typeof3.default)(this._rawEvent.read) === 'object';
    }

    /**
     * Determine if the event is a message delivery event.
     *
     */

  }, {
    key: 'isDelivery',
    get: function get() {
      return !!this._rawEvent.delivery && (0, _typeof3.default)(this._rawEvent.delivery) === 'object';
    }

    /**
     * Determine if the event is a postback or quick reply which includes payload.
     *
     */

  }, {
    key: 'isPayload',
    get: function get() {
      return !!this.postback && typeof this.postback.payload === 'string' || !!this.quickReply && typeof this.quickReply.payload === 'string';
    }

    /**
     * The payload received from postback or quick reply.
     *
     */

  }, {
    key: 'payload',
    get: function get() {
      if (!!this.postback && this.isPayload) {
        return this.postback.payload;
      } else if (!!this.quickReply && this.isPayload) {
        return this.quickReply.payload;
      }
      return null;
    }

    /**
     * Determine if the event is a policy enforcement event.
     *
     */

  }, {
    key: 'isPolicyEnforcement',
    get: function get() {
      return !!this._rawEvent['policy-enforcement'] && (0, _typeof3.default)(this._rawEvent['policy-enforcement']) === 'object';
    }

    /**
     * The policy enforcement object from Messenger raw event.
     *
     */

  }, {
    key: 'policyEnforcement',
    get: function get() {
      return this._rawEvent['policy-enforcement'] || null;
    }

    /**
     * Determine if the event is an app roles event.
     *
     */

  }, {
    key: 'isAppRoles',
    get: function get() {
      return !!this._rawEvent.app_roles && (0, _typeof3.default)(this._rawEvent.app_roles) === 'object';
    }

    /**
     * The app roles object from Messenger raw event.
     *
     */

  }, {
    key: 'appRoles',
    get: function get() {
      return this._rawEvent.app_roles || null;
    }

    /**
     * Determine if the event is a standby event.
     *
     */

  }, {
    key: 'isStandby',
    get: function get() {
      return this._isStandby;
    }

    /**
     * Determine if the event is a pass thread control event.
     *
     */

  }, {
    key: 'isPassThreadControl',
    get: function get() {
      return !!this._rawEvent.pass_thread_control && (0, _typeof3.default)(this._rawEvent.pass_thread_control) === 'object';
    }

    /**
     * The pass thread control object from Messenger raw event.
     *
     */

  }, {
    key: 'passThreadControl',
    get: function get() {
      return this._rawEvent.pass_thread_control || null;
    }

    /**
     * Determine if the event is a take thread control event.
     *
     */

  }, {
    key: 'isTakeThreadControl',
    get: function get() {
      return !!this._rawEvent.take_thread_control && (0, _typeof3.default)(this._rawEvent.take_thread_control) === 'object';
    }

    /**
     * The take thread control object from Messenger raw event.
     *
     */

  }, {
    key: 'takeThreadControl',
    get: function get() {
      return this._rawEvent.take_thread_control || null;
    }
  }]);
  return MessengerEvent;
}();

exports.default = MessengerEvent;