'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _delay = require('delay');

var _delay2 = _interopRequireDefault(_delay);

var _warning = require('warning');

var _warning2 = _interopRequireDefault(_warning);

var _messagingApiSlack = require('messaging-api-slack');

var _SlackEvent = require('./SlackEvent');

var _SlackEvent2 = _interopRequireDefault(_SlackEvent);

var _DelayableJobQueue = require('./DelayableJobQueue');

var _DelayableJobQueue2 = _interopRequireDefault(_DelayableJobQueue);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var SlackContext = function () {
  function SlackContext(_ref) {
    var client = _ref.client,
        event = _ref.event,
        session = _ref.session;
    (0, _classCallCheck3.default)(this, SlackContext);
    this._messageDelay = 1000;

    this._client = client;
    this._event = event;
    this._session = session;
    this._jobQueue = new _DelayableJobQueue2.default();
    this._jobQueue.beforeEach(function (_ref2) {
      var delay = _ref2.delay;
      return (0, _delay2.default)(delay);
    });
  }

  /**
   * The name of the platform.
   *
   */


  (0, _createClass3.default)(SlackContext, [{
    key: 'setMessageDelay',


    /**
     * Set delay before sending every messages.
     *
     */
    value: function setMessageDelay(milliseconds) {
      this._messageDelay = milliseconds;
    }

    /**
     * Delay and show indicators for milliseconds.
     *
     */

  }, {
    key: 'typing',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(milliseconds) {
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return (0, _delay2.default)(milliseconds);

              case 2:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function typing(_x) {
        return _ref3.apply(this, arguments);
      }

      return typing;
    }()
  }, {
    key: 'postMessage',
    value: function postMessage(text) {
      var _this = this;

      var channelId = this._getChannelIdFromSession();

      if (!channelId) {
        (0, _warning2.default)(false, 'postMessage: should not be called in context without session');
        return Promise.resolve();
      }

      return new Promise(function (resolve, reject) {
        _this._enqueue({
          instance: _this._client,
          method: 'postMessage',
          args: [channelId, text, { as_user: true }],
          delay: _this._messageDelay,
          showIndicators: true,
          onSuccess: resolve,
          onError: reject
        });
      });
    }

    /**
     * Send text to the owner of then session.
     *
     */

  }, {
    key: 'sendText',
    value: function sendText(text) {
      return this.postMessage(text);
    }
  }, {
    key: 'sendTextWithDelay',
    value: function sendTextWithDelay(delay, text) {
      var _this2 = this;

      (0, _warning2.default)(false, 'sendTextWithDelay is deprecated.');

      var channelId = this._getChannelIdFromSession();

      if (!channelId) {
        (0, _warning2.default)(false, 'sendTextWithDelay: should not be called in context without session');
        return Promise.resolve();
      }

      return new Promise(function (resolve, reject) {
        _this2._enqueue({
          instance: _this2._client,
          method: 'postMessage',
          args: [channelId, text, { as_user: true }],
          delay: delay,
          showIndicators: true,
          onSuccess: resolve,
          onError: reject
        });
      });
    }
  }, {
    key: '_enqueue',
    value: function _enqueue(job) {
      var _this3 = this;

      return new Promise(function (resolve, reject) {
        _this3._jobQueue.enqueue((0, _extends3.default)({}, job, {
          onSuccess: resolve,
          onError: reject
        }));
      });
    }

    // FIXME: this is to fix type checking

  }, {
    key: '_getChannelIdFromSession',
    value: function _getChannelIdFromSession() {
      if (!this._session) return null;
      if ((0, _typeof3.default)(this._session.channel) === 'object' && this._session.channel && this._session.channel.id && typeof this._session.channel.id === 'string') {
        return this._session.channel.id;
      }
      return null;
    }
  }, {
    key: 'platform',
    get: function get() {
      return 'slack';
    }

    /**
     * The client instance.
     *
     */

  }, {
    key: 'client',
    get: function get() {
      return this._client;
    }

    /**
     * The event instance.
     *
     */

  }, {
    key: 'event',
    get: function get() {
      return this._event;
    }

    /**
     * The session state of the context.
     *
     */

  }, {
    key: 'session',
    get: function get() {
      return this._session;
    }
  }]);
  return SlackContext;
}();

exports.default = SlackContext;