'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _pascalCase = require('pascal-case');

var _pascalCase2 = _interopRequireDefault(_pascalCase);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var SlackEvent = function () {
  function SlackEvent(rawEvent) {
    (0, _classCallCheck3.default)(this, SlackEvent);

    this._rawEvent = rawEvent;
  }

  /**
   * Underlying raw event from Slack.
   *
   */


  (0, _createClass3.default)(SlackEvent, [{
    key: 'rawEvent',
    get: function get() {
      return this._rawEvent;
    }

    /**
     * Determine if the event is a message event.
     *
     */

  }, {
    key: 'isMessage',
    get: function get() {
      return this._rawEvent.type === 'message';
    }

    /**
     * Determine if the event is a message event sent from channels.
     *
     */

  }, {
    key: 'isChannelsMessage',
    get: function get() {
      if (!this.isMessage) return false;

      var message = this.message;

      return message.channel.startsWith('C');
    }

    /**
     * Determine if the event is a message event sent from groups.
     *
     */

  }, {
    key: 'isGroupsMessage',
    get: function get() {
      if (!this.isMessage) return false;

      var message = this.message;

      return message.channel.startsWith('G');
    }

    /**
     * Determine if the event is a message event sent from instant messaging.
     *
     */

  }, {
    key: 'isImMessage',
    get: function get() {
      if (!this.isMessage) return false;

      var message = this.message;

      return message.channel.startsWith('D');
    }

    /**
     * Determine if the event is a message event sent from multiple people instant messaging.
     *
     */

  }, {
    key: 'isMpimMessage',
    get: function get() {
      if (!this.isMessage) return false;

      var message = this.message;

      return message.channel.startsWith('G');
    }

    /**
     * The message object from Slack raw event.
     *
     */

  }, {
    key: 'message',
    get: function get() {
      if (!this.isMessage) return;

      var message = this._rawEvent;

      return message;
    }

    /**
     * Determine if the event is a message event which includes text.
     *
     */

  }, {
    key: 'isTextMessage',
    get: function get() {
      return this.isMessage;
    }
  }]);
  return SlackEvent;
}();

// https://api.slack.com/events


exports.default = SlackEvent;
var eventTypes = ['app_uninstalled', 'channel_archive', 'channel_created', 'channel_deleted', 'channel_history_changed', 'channel_rename', 'channel_unarchive', 'dnd_updated', 'dnd_updated_user', 'email_domain_changed', 'emoji_changed', 'file_change', 'file_comment_added', 'file_comment_deleted', 'file_comment_edited', 'file_created', 'file_deleted', 'file_public', 'file_shared', 'file_unshared', 'grid_migration_finished', 'grid_migration_started', 'group_archive', 'group_close', 'group_history_changed', 'group_open', 'group_rename', 'group_unarchive', 'im_close', 'im_created', 'im_history_changed', 'im_open', 'link_shared', 'member_joined_channel', 'member_left_channel', 'pin_added', 'pin_removed', 'reaction_added', 'reaction_removed', 'star_added', 'star_removed', 'subteam_created', 'subteam_members_changed', 'subteam_self_added', 'subteam_self_removed', 'subteam_updated', 'team_domain_change', 'team_join', 'team_rename', 'tokens_revoked', 'url_verification', 'user_change'];

eventTypes.forEach(function (event) {
  Object.defineProperty(SlackEvent.prototype, 'is' + (0, _pascalCase2.default)(event), {
    enumerable: false,
    configurable: true,
    get: function get() {
      return this._rawEvent.type === event;
    }
  });
});