'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _delay = require('delay');

var _delay2 = _interopRequireDefault(_delay);

var _warning = require('warning');

var _warning2 = _interopRequireDefault(_warning);

var _messagingApiTelegram = require('messaging-api-telegram');

var _TelegramEvent = require('./TelegramEvent');

var _TelegramEvent2 = _interopRequireDefault(_TelegramEvent);

var _DelayableJobQueue = require('./DelayableJobQueue');

var _DelayableJobQueue2 = _interopRequireDefault(_DelayableJobQueue);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var TelegramContext = function () {
  function TelegramContext(_ref) {
    var client = _ref.client,
        event = _ref.event,
        session = _ref.session;
    (0, _classCallCheck3.default)(this, TelegramContext);
    this._messageDelay = 1000;

    this._client = client;
    this._event = event;
    this._session = session;
    this._jobQueue = new _DelayableJobQueue2.default();
    this._jobQueue.beforeEach(function (_ref2) {
      var delay = _ref2.delay;
      return (0, _delay2.default)(delay);
    });
  }

  /**
   * The name of the platform.
   *
   */


  (0, _createClass3.default)(TelegramContext, [{
    key: 'setMessageDelay',


    /**
     * Set delay before sending every messages.
     *
     */
    value: function setMessageDelay(milliseconds) {
      this._messageDelay = milliseconds;
    }

    /**
     * Delay and show indicators for milliseconds.
     *
     */

  }, {
    key: 'typing',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(milliseconds) {
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return (0, _delay2.default)(milliseconds);

              case 2:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function typing(_x) {
        return _ref3.apply(this, arguments);
      }

      return typing;
    }()

    /**
     * Send text to the owner of then session.
     *
     */

  }, {
    key: 'sendText',
    value: function sendText(text) {
      if (!this._session) {
        (0, _warning2.default)(false, 'sendText: should not be called in context without session');
        return Promise.resolve();
      }
      return this._enqueue({
        instance: this._client,
        method: 'sendMessage',
        args: [this._session.user.id, text],
        delay: this._messageDelay,
        showIndicators: true
      });
    }
  }, {
    key: 'sendTextWithDelay',
    value: function sendTextWithDelay(delay, text) {
      (0, _warning2.default)(false, 'sendTextWithDelay is deprecated.');

      if (!this._session) {
        (0, _warning2.default)(false, 'sendText: should not be called in context without session');
        return Promise.resolve();
      }
      return this._enqueue({
        instance: this._client,
        method: 'sendMessage',
        args: [this._session.user.id, text],
        delay: delay,
        showIndicators: true
      });
    }
  }, {
    key: '_enqueue',
    value: function _enqueue(job) {
      var _this = this;

      return new Promise(function (resolve, reject) {
        _this._jobQueue.enqueue((0, _extends3.default)({}, job, {
          onSuccess: resolve,
          onError: reject
        }));
      });
    }
  }, {
    key: 'platform',
    get: function get() {
      return 'telegram';
    }

    /**
     * The client instance.
     *
     */

  }, {
    key: 'client',
    get: function get() {
      return this._client;
    }

    /**
     * The event instance.
     *
     */

  }, {
    key: 'event',
    get: function get() {
      return this._event;
    }

    /**
     * The session state of the context.
     *
     */

  }, {
    key: 'session',
    get: function get() {
      return this._session;
    }
  }]);
  return TelegramContext;
}();

var sendMethods = ['sendMessage', 'sendPhoto', 'sendAudio', 'sendDocument', 'sendSticker', 'sendVideo', 'sendVoice',
// 'sendVideoNote',
'sendLocation', 'sendVenue', 'sendContact', 'sendChatAction'];

sendMethods.forEach(function (method) {
  Object.defineProperty(TelegramContext.prototype, '' + method, {
    enumerable: false,
    configurable: true,
    writable: true,
    value: function value() {
      if (!this._session) {
        (0, _warning2.default)(false, method + ': should not be called in context without session');
        return Promise.resolve();
      }

      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      return this._enqueue({
        instance: this._client,
        method: method,
        args: [this._session.user.id].concat(args),
        delay: this._messageDelay,
        showIndicators: true
      });
    }
  });

  Object.defineProperty(TelegramContext.prototype, method + 'WithDelay', {
    enumerable: false,
    configurable: true,
    writable: true,
    value: function value(delay) {
      if (!this._session) {
        (0, _warning2.default)(false, method + 'WithDelay: should not be called in context without session');
        return Promise.resolve();
      }

      for (var _len2 = arguments.length, rest = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
        rest[_key2 - 1] = arguments[_key2];
      }

      return this._enqueue({
        instance: this._client,
        method: method,
        args: [this._session.user.id].concat(rest),
        delay: delay,
        showIndicators: true
      });
    }
  });
});

exports.default = TelegramContext;