'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _restify = require('restify');

var _restify2 = _interopRequireDefault(_restify);

var _shortid = require('shortid');

var _shortid2 = _interopRequireDefault(_shortid);

var _connectNgrok = require('../connectNgrok');

var _connectNgrok2 = _interopRequireDefault(_connectNgrok);

var _createMiddleware = require('./createMiddleware');

var _createMiddleware2 = _interopRequireDefault(_createMiddleware);

var _verifyLINESignature = require('./verifyLINESignature');

var _verifyLINESignature2 = _interopRequireDefault(_verifyLINESignature);

var _verifyMessengerWebhook = require('./verifyMessengerWebhook');

var _verifyMessengerWebhook2 = _interopRequireDefault(_verifyMessengerWebhook);

var _verifySlackWebhook = require('./verifySlackWebhook');

var _verifySlackWebhook2 = _interopRequireDefault(_verifySlackWebhook);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createServer(bot) {
  var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var path = config.path || '/';
  var verifyToken = void 0;
  var server = _restify2.default.createServer();

  server.use(_restify2.default.plugins.queryParser());
  server.use(_restify2.default.plugins.bodyParser());
  if (bot.connector.platform === 'messenger') {
    verifyToken = config.verifyToken || _shortid2.default.generate();
    server.get(path, (0, _verifyMessengerWebhook2.default)({ verifyToken: verifyToken }));
    server.post(path, (0, _createMiddleware2.default)(bot));
  } else if (bot.connector.platform === 'slack') {
    server.post(path, (0, _verifySlackWebhook2.default)(), (0, _createMiddleware2.default)(bot));
  } else if (bot.connector.platform === 'line') {
    server.post(path, (0, _verifyLINESignature2.default)(bot), (0, _createMiddleware2.default)(bot));
  } else {
    server.post(path, (0, _createMiddleware2.default)(bot));
  }

  var _listen = server.listen.bind(server);
  server.listen = function () {
    _listen.apply(undefined, arguments);
    if (config.ngrok) {
      (0, _connectNgrok2.default)(arguments.length <= 0 ? undefined : arguments[0], function (err, url) {
        console.log('webhook url: ' + url + path);
      });
    }
    if (bot.connector.platform === 'messenger') {
      console.log('verify token: ' + verifyToken);
    }
  };

  return server;
}

exports.default = createServer;